/*
 * mscodecs.h
 *
 * Microsoft nonstandard codecs handler
 *
 * Open H323 Library
 *
 * Copyright (c) 1998-2000 Equivalence Pty. Ltd.
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 *
 * The Original Code is Open H323 Library.
 *
 * The Initial Developer of the Original Code is Equivalence Pty. Ltd.
 *
 * Contributor(s): ______________________________________.
 *
 * $Log: mscodecs.h,v $
 * Revision 1.1  2000/08/23 14:23:11  craigs
 * Added prototype support for Microsoft GSM codec
 *
 *
 */

#ifndef __MSCODECS_H
#define __MSCODECS_H

#include "h323caps.h"

class MicrosoftNonStandardAudioCapability : public H323AudioCapability,
                                            public H323NonStandardCapabilityInfo
{
  PCLASSINFO(MicrosoftNonStandardAudioCapability, H323AudioCapability);

  public:
    MicrosoftNonStandardAudioCapability(unsigned maxPacketSize,      /// Maximum size of an audio packet in bytes
                                        unsigned desiredPacketSize,  /// Desired transmit size of an audio packet
                                        const BYTE * header,         //  Non standard data
                                        PINDEX headerSize,
                                        PINDEX offset,
                                        PINDEX len);

    unsigned GetSubType() const;

    BOOL OnSendingPDU(H245_AudioCapability & pdu,
                      unsigned packetSize) const;

    BOOL OnReceivedPDU(const H245_AudioCapability & pdu,
                       unsigned & packetSize);
};

/////////////////////////////////////////////////////////////////////////

class MicrosoftGSMAudioCapability : public MicrosoftNonStandardAudioCapability
{
  PCLASSINFO(MicrosoftGSMAudioCapability, MicrosoftNonStandardAudioCapability);

  public:
    MicrosoftGSMAudioCapability();
    PObject * MicrosoftGSMAudioCapability::Clone() const;
    PString MicrosoftGSMAudioCapability::GetFormatName() const;
    H323Codec * MicrosoftGSMAudioCapability::CreateCodec(H323Codec::Direction direction) const;
};

/////////////////////////////////////////////////////////////////////////

class MicrosoftGSMCodec : public H323FramedAudioCodec
{
  PCLASSINFO(MicrosoftGSMCodec, H323FramedAudioCodec);
  public:

  /**@name Construction */
  //@{
    /**Create a new GSM 06.10 codec for ALaw.
     */
    MicrosoftGSMCodec(Direction direction, /// Direction in which this instance runs
                   RTP_DataFrame::PayloadTypes rtpType
    );

    ~MicrosoftGSMCodec();
  //@}

    /**Encode a sample block into the buffer specified.
       The samples have been read and are waiting in the readBuffer member
       variable. it is expected this function will encode exactly
       encodedBlockSize bytes.
     */
    virtual BOOL EncodeFrame(
      BYTE * buffer,    /// Buffer into which encoded bytes are placed
      unsigned & length /// Actual length of encoded data buffer
    );

    /**Decode a sample block from the buffer specified.
       The samples must be placed into the writeBuffer member variable. It is
       expected that no more than frameSamples is decoded. The return value
       is the number of samples decoded. Zero indicates an error.
     */
    virtual BOOL DecodeFrame(
      const BYTE * buffer,  /// Buffer from which encoded data is found
      unsigned length,      /// Length of encoded data buffer
      unsigned & written    /// Number of bytes used from data buffer
    );

  protected:
    struct gsm_state * gsm;
};

/////////////////////////////////////////////////////////////////////////

#endif
